#include <curl/curl.h>
#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include "emptiness.h"
#include <sstream>
#include <string>
using namespace std;
struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

Input
input_data(istream& tin, bool prompt)
{
    size_t number_count;
    if (prompt)
        cerr << "Enter number count: ";
    tin >> number_count;
    Input in;
    in.numbers.resize(number_count);
    for(int i; i<number_count; i++)
    {
        tin >> in.numbers[i];
    }
    if (prompt)
        cerr << "Enter bin count: ";
    tin >> in.bin_count;
    return in;
}

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx)
{
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input
download(const string& address)
{
    stringstream buffer;
    CURL *curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK)
        {
            fprintf(stderr, "curl_easy_perform() failed: %s\n",curl_easy_strerror(res));
            exit(1);
        }
        curl_easy_cleanup(curl);

    }
    return input_data(buffer, false);
}

int
main(int argc, char* argv[])
{
    Input input;
    if (argc > 1)
    {
        input = download(argv[1]);
    }
    else
    {
        input = input_data(cin, true);
    }

    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
}

